%%%%%%%%%%%%%%%%%%%%%%%%%%
% https://www.iriscores.com
% vojta.ters@iriscores.com
% 2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
clear all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% GET DISPLAY RESOLUTION %%%%%%%
Displ = groot();
ScW = Displ.ScreenSize(3);
ScH = Displ.ScreenSize(4);
FiW = 1440;
FiH = 920;
Cls = perms([0,102,204]./255);
Cl_Brown      = 1;
Cl_Dark_Red   = 2;
Cl_Dark_Green = 3;
Cl_Violet     = 4;
Cl_Green      = 5;
Cl_Blue       = 6; 
LW            = 2;
Fs            = 13;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add release Directory to Matlab Path %
MexFilePath = ['..',filesep,'DaySim',filesep,'x64',filesep,'Release'];
addpath(MexFilePath);


ModelDownScale = 1000;    % DownScale All Computing so that Numeric Errors are suppressed   
TimeReduce = 1;           % Divide total amount of seconds per year by this value (Reduce Discretization)
Days = 365;               % Amount of Days Per Year
Earth_A = 149.60e9;       % Meters (Semi Major Earth Orbital Axis)
Earth_R = 6378e3;         % Earth Radius in Meters
EarthTilt = 23.43;        % Earth Tilt in Degrees 
UseCUDA   = 1;            % Compute Via nVidia CUDA
UseCUDASP = 0;            % Use Single Precision for CUDA (Not Recommended,Although Works With DownScale 1000x)
Lat  = 49.963051;         % "Your" / Observer Latitude 
Long = 14.803021;         % "Your" / Observer Longitude
SunCenter = [0;0;0];      % Simulation Suggest the Sun is the center of the universe
ShowSample = 1;           % Show Sample Visualization
PolarNight = 0;           % User location is not influenced by polar night/day
Earth_A = Earth_A / ModelDownScale;

Months = {'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Nov','Oct','Dec'};
Months = circshift(Months,[1,3]);
MonthDays = 365/12;
XData = MonthDays/2:MonthDays:365;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create Earth Orbital Trajectory (Ellipse)
% Note: This simulation will ignore the Eccentricity and will continue to use circular trajectory
%Eccentricity = 0.0167086; % Eccentricity of the Earth
%Earth_B = Earth_A*sqrt(1 - Eccentricity^2);
%a = Earth_A;
%b = Earth_B;
%r = Earth_R;
%x = linspace(-Earth_A,Earth_A,Days);
%y = sqrt((a.*a.*b.*b -b.*b.*x.*x)/(a.*a));

if(ShowSample == 1)
    % Create Earth Sphere
    EarhPrecision = 1000;
    [Ex,Ey,Ez] = sphere(EarhPrecision);
    Exx = Ex * Earth_R;
    Eyy = Ey * Earth_R;
    Ezz = Ez * Earth_R;

    %%%%%%%%%%%%%%%%%%%%%%%%
    % Create Equator plane %
    step = Earth_R/100;
    [px,py] = meshgrid(-1.5*Earth_R:step:1.5*Earth_R); % Generate x and y data
    pz = zeros(size(px, 1));                           % Generate z data

    %%%%%%%%%%%%%%%%%%%%
    % Create Axis Line %
    Ax = zeros(1,100);
    Ay = zeros(1,100);
    Az = linspace(-1.5*Earth_R,1.5*Earth_R,100);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Visualize your Position %
    [xx,yy,zz] = sph2cart(deg2rad(Long),deg2rad(Lat),Earth_R);
    Pos = [xx;yy;zz];
    NewPosZ = RotateZ(Pos,EarthTilt);
    NewPosY = RotateY(Pos,EarthTilt);
    NewPosX = RotateX(Pos,EarthTilt);

    SphereAxis  = [Ax;Ay;Az];
    EarthAxis   = RotateX(SphereAxis,EarthTilt);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% COMPUTE A STATIC POSITION ON EARTH IN TIME %%%%%%%
    NumPoints = 24;
    Rotations = linspace(0,360,NumPoints);
    ObserverPositions = zeros(3,NumPoints);
    for i = 1:NumPoints

        CAngle = Rotations(i);
        P = Pos;

        P = RotateZ(P,CAngle);     % EG Time Rotation
        P = RotateX(P,EarthTilt);  % Earth Tilt

        ObserverPositions(:,i) = P;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% Rotate The Earth shape by the tilt for better visualization %%%%%%%%
    % Note: This is only to better visualize the Image Mapping, Rotating Sphere otherwice doesnt make sense
    Earth = zeros([size(Exx),3]);
    Earth(:,:,1) = Exx;
    Earth(:,:,2) = Eyy;
    Earth(:,:,3) = Ezz;
    Earth = RotateX(Earth,EarthTilt);
    Exx = squeeze(Earth(:,:,1));
    Eyy = squeeze(Earth(:,:,2));
    Ezz = squeeze(Earth(:,:,3));


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% Display Erath with Equatorial Plane and Orbiting Axis %%%%%%%
    Im = im2double(imread(['Images',filesep,'Earth_Map.jpg']));
    figure('name','Earth orbit','position',[ScW/2-FiW/2,ScH/2-FiH/2,FiW,FiH]);
    % Earth's Orbital Trajectory is almost a circle
    %subplot(3,1,1);
    %plot(x,y,'color',Cl1);
    %hold on;grid on;
    %plot(x,-y,'color',Cl1);
    %plot(0,0,'.','MarkerSize',22,'color','k');
    %title('Earth''s orbit trajectory around the sun');
    %axis equal;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Display Earth Rotation around its Axis with respect to Receiver Position %
    subplot(3,1,[1,2,3]);
    surf(Exx,Eyy,Ezz,'FaceColor','texturemap','EdgeColor','none','Cdata',flipud(Im));
    hold on;grid on;
    surf(px,py,pz,'edgecolor','none','facecolor','k','facealpha',0.5);
    line(Ax,Ay,Az,'LineWidth',5,'color','b');
    line(EarthAxis(1,:),EarthAxis(2,:),EarthAxis(3,:),'linewidth',5,'color',Cls(Cl_Dark_Red,:));
    title('Earth Visualization');


    c = 0.05;        % EarhtSize
    SpherePrec = 10; % Precision of Receiver Pos
    for i = 1:NumPoints
        Px = ObserverPositions(1,i);
        Py = ObserverPositions(2,i);
        Pz = ObserverPositions(3,i); 
        [X,Y,Z] = sphere(SpherePrec);
        X = c*Earth_R*X + Px;
        Y = c*Earth_R*Y + Py;
        Z = c*Earth_R*Z + Pz;

        surf(X,Y,Z,'edgecolor','none','facecolor',Cls(Cl_Green,:));
    end

    Ax = gca;
    Ax.Box = 'on';
    Ax.BoxStyle = 'full';

    Ax.YLim = [-1.5*Earth_R,1.5*Earth_R];
    Ax.XLim = [-1.5*Earth_R,1.5*Earth_R];
    Ax.ZLim = [-1.5*Earth_R,1.5*Earth_R];
    axis equal;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the Visibility %
DaySec = 24*60*60/TimeReduce;                 % Amount of Seconds per Day
Seconds = DaySec*Days;                        % Amount of Seconds per Year
Visible = zeros(1,Seconds);                   % Initialize Array with visibility
Time = 0:Seconds-1;                           % Time of the Year in seconds
AngleRot = mod(Time,DaySec)./DaySec * 360;    % Rotation of the Earth in Time in Degrees

% For Simplicity,Assume the Earth movement is only in the XY Plane
% This should hava no real impact on the final results due to symmetry
OrbitRot = 0:1/Seconds:1-1/Seconds;
OrbitRot = OrbitRot * 360;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% COMPUTE POSITION OF THE EARTH IN TIME %%%%%%%
Ep  = zeros(3,Seconds);
MyP = zeros(3,Seconds);
Ep(1,:) = Earth_A * cos(deg2rad(OrbitRot));
Ep(2,:) = Earth_A * sin(deg2rad(OrbitRot));
Ep(3,:) = zeros(1,Seconds);
[x,y,z]   = sph2cart(deg2rad(Long),deg2rad(Lat),Earth_R);
MyPos = [x;y;z];

if(UseCUDA == 1)
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if(UseCUDASP == 1) %%%%%%%% Single Precision CUDA Computation %%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Note: Single Precision Might not Be sufficient!
        % Computing in Meters, Earth Orbital Diameter ^2 = approx 1e18
        % This is half of the single precision limit.
        warning('Please consider using Double Precision for Astronomical computations!');
        EarthPosS  = single(Ep);
        AngleRotS  = single(AngleRot);
        SunCenterS = single(SunCenter);
        LocationS  = single(MyPos);
        ParamsS    = single([EarthTilt,Earth_R]);
        tic
        Visible = DaySim(EarthPosS,AngleRotS,SunCenterS,LocationS,ParamsS);
        toc
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    else %%%%%%% Double Precision CUDA Computation %%%%%%%% 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % This is the recommended computing mode
        EarthPosD  = double(Ep);
        AngleRotD  = double(AngleRot);
        SunCenterD = double(SunCenter);
        LocationD  = double(MyPos);
        ParamsD    = double([EarthTilt,Earth_R]);
        tic
        Visible = DaySim(EarthPosD,AngleRotD,SunCenterD,LocationD,ParamsD);
        toc
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else %%%%%%% Double Precision Matlab "CPU" Computing %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % NOTE: Make sure to Reduce amount of point by at least 10 times
    % So that you dont have to wait ~10 minutes on the result.
    tic
    
    for i  = 1:Seconds
        MyP(:,i)   = RotateZ(MyPos,AngleRot(i));     % EG Time Rotation
        MyP(:,i)   = RotateX(MyP(:,i),EarthTilt);    % Earth Axis Rotation
    end

    % Include Earth Position
    MyP = MyP + Ep;

    Day = 1;
    for i = 1:Seconds
        if(mod(i-1,DaySec) == (DaySec -1))
            disp(['Processing Day: ',num2str(Day)]);
            Day = Day + 1;
        end


        % COMPUTE ROTATION OF THE POINT IN TIME
        %EarthPos = [Xe(i);Ye(i);Ze(i)];
        %MyP   = RotateZ(MyPos,AngleRot(i));     % EG Time Rotation
        %MyP   = RotateX(MyP,EarthTilt);         % Earth Axis Rotation
        %MyP = MyP + EarthPos;

        
        Visible(i) = SunVisible( MyP(:,i),SunCenter,Ep(:,i),Earth_R );
    end
    toc
end

clear mex;



%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% POST PROCESS THE DATA TO GET THE OVERVIEW OF THE SITUATION %%%%%%%
if(PolarNight == 1)
    DayTime = zeros(1,Days);
    Day = 0;
    Vis = 0;
    DayEnded = 1;
    DayStarted = 0;
    for  i = 2:Seconds - 1
        
        
        if((Visible(i) == 1) && (Visible(i-1) == 0) && (DayEnded == 1) && (DayStarted == 0))
            Day = Day + 1;
            DayEnded = 0;
            DayStarted = 1;
        end
        
        if((Visible(i) == 1) && (DayStarted == 1))
            Vis = Vis + 1;

        end
        
        
        if((Visible(i) == 1) && (Visible(i+1) == 0))
            
            if(Vis > 500)
                disp(['Day: ',num2str(Day),' Seconds: ',num2str(Vis)]);
                DayTime(Day) = Vis;
                Vis = 0;
                DayEnded   = 1;
                DayStarted = 0;
            end
        end
        
    end
    
else
    % This Algorithm Searches for Amount of Daylight by removing night.
    % If the Night (Polar Night) is too long,it will not find 365 values.
    Ind = 1;
    Days = 365;
    DayTime = zeros(1,Days);
    for i = 1:Days

        % Get Rid of Night
        Vis = 0;
        while(1)
            if(Visible(Ind) == 0)
                Ind = Ind + 1;
            else
                break;
            end
        end

        % Count Seconds when the Sun is Visible
        while(1)
            if(Visible(Ind) == 1)
                Vis = Vis + 1;
            else
                break;
            end
            Ind = Ind + 1;
        end

        DayTime(i) = Vis;
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate Difference of Daylight Between Days %
GR = gradient(DayTime);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% DISPLAY RESULTS %%%%%%%
figure('name','Visualization of the Total DayLight','position',[ScW/2-FiW/2,ScH/2-FiH/2,FiW,FiH]);
subplot(3,1,[1,2]);
plot(DayTime./60*TimeReduce,'color',Cls(Cl_Blue,:),'linewidth',LW);
hold on;grid on;
ylabel('Total DayLight [Minutes]','FontSize',Fs,'Color',Cls(Cl_Violet,:),'FontName','Arial','FontWeight','bold');
title(['Amount of Daylight on Latitude: ',num2str(Lat),' & Longitude: ',num2str(Long)]);
xlim([0,length(DayTime)]);
ax = gca;
for i = 1:12
    ax.XTick(i) = XData(i);
    ax.XTickLabel{i} = Months{i};
end


subplot(3,1,3);
plot(GR./60*TimeReduce,'color',Cls(Cl_Dark_Red,:),'linewidth',LW);
hold on;grid on;
ylabel('DayLight Difference [Minutes]','FontSize',Fs,'Color',Cls(Cl_Violet,:),'FontName','Arial','FontWeight','bold');
title('DayLight Difference');
xlim([0,length(DayTime)]);
ax = gca;
for i = 1:12
    ax.XTick(i) = XData(i);
    ax.XTickLabel{i} = Months{i};
end







