%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%http://www.homer.com.au/webdoc/geometry/sphereline.htm
% @brief Check if the Sun is Visible from the Given Point
% @param SunCenter  : Is Expected to be [0,0,0],but May Vary
% @param EarhCenter : Center of the Earth [X,Y,Z] Corrdinates 
% @param Earth Radius : 6378e3 Likely
% https://www.iriscores.com
% vojta.ters@iriscores.com
% 2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ Visible ] = SunVisible( Point,SunCenter,EarthCenter,EarthRadius )

Visible = 1;


a = sum((SunCenter - Point).^2);
b = 2*sum((SunCenter - Point).*(Point - EarthCenter));
c = sum(EarthCenter.^2) + sum((Point.^2)) - 2*(sum(EarthCenter.*Point)) - EarthRadius.^2;
Res = b * b - 4*a*c;



if(Res < 0)
    % This Is unlikely to happen
    %disp('Please check');
end

if(Res == 0)
    % Either Sunset or Sunrise
    disp('Sunrise / Sunset');
end

if(Res > 0)
    % Intersection through the Earth
    R0 = (-b + sqrt(Res)) / (2*a);
    R1 = (-b - sqrt(Res)) / (2*a);
    
    IP0 = Point + R0 .* (SunCenter - Point);
    IP1 = Point + R1 .* (SunCenter - Point);
    
    % Have to Check whether the receiver is further from the sun - Night
    % Or whether the Intersection Point is closer to the Sun - Day
    D0 = sqrt(sum((IP0 - SunCenter).^2));
    D1 = sqrt(sum((IP1 - SunCenter).^2));
    DP = sqrt(sum((Point - SunCenter).^2));
    

    Diff0 = abs(D0 - DP);
    Diff1 = abs(D1 - DP);
    if(Diff0 < Diff1)

        if(D0 < D1)
            % Receiver is at P0 and is Closer to the Sun
            % The Sun should be visible
            Visible = 1;
        else
            % The Receiver is at P0,but this is further from the Sun
            % The Sun is not Visible
            Visible = 0;
        end
    else

        if(D1 < D0)
            % The Receiver is at P1 and is Closer to the Sun
            % The Sun is visible
            Visible = 1;
        else
            % The Receiver is at P1,but is further from the Sun
            % The Sun is not Visible
            Visible = 0;
        end
    end
    
end


end

